var termBlaster_SearchWindow;

/**
 * Observer that updates the windows whenever the options have been changed
 */
function termBlasterOptionsObserver()
{
  this.register();
}
termBlasterOptionsObserver.prototype = {
  observe: function(subject, topic, data) {
    TermBlaster.updateMenu();
  },

  register: function() {
    var observerService = Components.classes["@mozilla.org/observer-service;1"]
                          .getService(Components.interfaces.nsIObserverService);
    observerService.addObserver(this, "TermBlasterOptions_btnOkCommand", false);
  },

  unregister: function() {
    var observerService = Components.classes["@mozilla.org/observer-service;1"]
                          .getService(Components.interfaces.nsIObserverService);
    observerService.removeObserver(this, "TermBlasterOptions_btnOkCommand");
  }
};

var TermBlaster = {
  onLoad: function() {
    document.getElementById("contentAreaContextMenu")
      .addEventListener("popupshowing", TermBlaster.onPopup, false);

    TermBlaster.updateMenu();
    try {
      this.observer = new termBlasterOptionsObserver();
    } catch (e) {
      alert("termBlaster could not register termBlaster Options window's 'Ok' button observer: " + e.message);
    }
  },

  onUnLoad: function() {
    try {
      this.observer.unregister();
    } catch (e) {
      alert("termBlaster could not unregister termBlaster Options window's 'Ok' button observer: " + e.message);
    }
  },

  onMenuItemCommand: function() {
    window.open("chrome://termblaster/content/options.xul", "termBlaster_settings", "chrome");
  },

  /*/
   * Executes whenever the context menu is opened
   */
  onPopup: function() {
    var database_popup = document.getElementById("termblaster-database-popup");
    var database_menu = document.getElementById("termblaster-database-menu");
    var quicklist_popup = document.getElementById("termblaster-quicklist-popup");
    var quicklist_menu = document.getElementById("termblaster-quicklist-menu");
    var seperator = document.getElementById("termblaster-separator");
    var textSelected = gContextMenu.isTextSelected;

    database_menu.hidden = !database_popup.hasChildNodes() || !textSelected;
    quicklist_menu.hidden = !quicklist_popup.hasChildNodes() || !textSelected;

    seperator.hidden = (database_menu.hidden && quicklist_menu.hidden);
  },

  updateMenu: function() {
    try {
      TermBlasterPopulator.loadPopup('database', document.getElementById("termblaster-database-popup"));
      TermBlasterPopulator.loadPopup('quicklist', document.getElementById("termblaster-quicklist-popup"));
    } catch (e) {
      alert("termBlaster could not create popup menus: " + e.message);
    }
  },

  /**
   * Searches the selected text, with the method set in the extension's options
   * 
   * @params sQuery The query string of the selected search engine
   */
  search: function (sQuery) {
    var selection = document.commandDispatcher.focusedWindow.getSelection()
      .toString().replace(/^\s+|\s+$/g,"");

    if (selection.length < 1)
      return;

    sQuery = sQuery.replace('\$s', encodeURIComponent(selection));

    try {
      var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                                getService(Components.interfaces.nsIPrefBranch);

      if (prefs.getPrefType("extensions.termblaster.destination") != prefs.PREF_INT)
        prefs.clearUserPref("extensions.termblaster.destination");

      switch (prefs.getIntPref("extensions.termblaster.destination")) {
        // Use only a single window with a dirty little trick
        case 0:
          try {
            termBlaster_SearchWindow.addTab(sQuery);
          } catch (e) {
            window.open(sQuery);
            var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                   .getService(Components.interfaces.nsIWindowMediator);
            var newWindow = wm.getMostRecentWindow("navigator:browser");
            termBlaster_SearchWindow = newWindow.getBrowser();
          }
        break;
        // Open result in a new window
        case 2: window.open(sQuery);
        break;
        // Open results in current window
        case 3: getBrowser().loadURI(sQuery);
        break;
        // Open results in a new tab of current window
        default: getBrowser().addTab(sQuery);
      }
    } catch (e) {
      alert("termBlaster could not execute search: " + e.message);
    }
  }
};

var TermBlasterPopulator = {
  XUL_NS: "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul",
  loadPopup: function(key, popup) {
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefBranch);
    var uri = prefs.getCharPref("extensions.termblaster.opened." + key).replace(/^\s*|\s*$/g, "");
    if (uri == "chrome://termblaster/content/Services.tbd"
     || uri == "chrome://termblaster/content/QuickList.tbq") {
      prefs.clearUserPref("extensions.termblaster.opened." + key);
      uri = prefs.getCharPref("extensions.termblaster.opened." + key).replace(/^\s*|\s*$/g, "");
    }

    if (uri == "") {
      popup.parentNode.hidden = true;
      return false;
    }

    // Executes after the XML file has been loaded
    var XMLDoc = document.implementation.createDocument("", "", null);
    XMLDoc.async = false;
    XMLDoc.onload = function () {
      while (popup.hasChildNodes()) {
        popup.removeChild(popup.firstChild);
      }

      TermBlasterPopulator.loadNode(XMLDoc.documentElement, popup);
    }

    if (!popup.hasChildNodes()) {
      popup.parentNode.hidden = true;
    }

    // Load an XML file
    try {
      XMLDoc.load(uri);
    } catch (e) {
      alert("termBlaster could not load selected file ('" + uri + "'): " + e.message);
    }
  },

  loadNode: function (iNode, popup) {
    if (iNode != null && iNode.localName != null) {
      if (iNode.localName == 'service') {  // Process a leaf node
        this.appendMenuItem(iNode.attributes.getNamedItem("name").nodeValue, iNode.attributes.getNamedItem("query").nodeValue, popup);
      } else {
        if (iNode.localName == 'folder') {  // Process a folder node
          popup = this.appendMenu(iNode.attributes.getNamedItem("name").nodeValue, popup);
        }

        // Load all child nodes
        var cNode;
        for (var i = 0; i < iNode.childNodes.length; i++) {
          cNode = iNode.childNodes.item(i);
          if (cNode.localName != null) {
            this.loadNode(cNode, popup);
          }
        }

        if (!popup.parentNode.hidden && !popup.hasChildNodes()) {
          popup.parentNode.hidden = true;
        }
      }
    } 
  },

  appendMenu: function(aLabel, popup) {
    var menu = document.createElementNS(this.XUL_NS, "menu");
    menu.setAttribute("class", "menu-iconic termblaster-icon-folder");
    menu.setAttribute("label", aLabel);
    var menupopup = document.createElementNS(this.XUL_NS, "menupopup");
    menu.appendChild(menupopup);
    popup.appendChild(menu);

    return menupopup;
  },

  appendMenuItem: function(aLabel, aQuery, popup) {  
    var item = document.createElementNS(this.XUL_NS, "menuitem");
    item.setAttribute("class", "menuitem-iconic termblaster-icon-file");
    item.setAttribute("label", aLabel);
    item.setAttribute("oncommand", "TermBlaster.search('" + aQuery + "');");
    popup.appendChild(item);

    return item;
  }
}
